<?php

class CommerceHelper {
  /**
   * Return cart order wrapper by user id.
   * @param integer|NULL $uid User id or NULL for use current user id
   * @return EntityMetadataWrapper|NULL
   */
  public static function getCartOrderWrapper($uid = NULL) {
    if ($uid === NULL) {
      $uid = $GLOBALS['user']->uid;
    }
    if ($cart_order = commerce_cart_order_load($uid)) {
      return entity_metadata_wrapper('commerce_order', $cart_order);
    }
  }

  /**
   * Return cart product ids.
   * @param integer|NULL $uid User id or NULL for use current user id
   * @return array
   */
  public static function getCartOrderProductIds($uid = NULL) {
    $cart_order_wrapper = self::getCartOrderWrapper($uid);
    return self::getOrderProductIds($cart_order_wrapper);
  }

  /**
   * Return TRUE if cart is empty.
   * @return boolean
   */
  public static function cartIsEmpty() {
    return self::getCartQuantity() == 0;
  }

  /**
   * Return order product ids.
   * @param integer|stdClass|EntityMetadataWrapper $order Order id, or object, or wrapper
   * @return array
   */
  public static function getOrderProductIds($order) {
    $order = EntityHelper::getEntityWrapper($order, 'commerce_order');
    $product_ids = array();
    $line_item_types = commerce_product_line_item_types();
    foreach ($order->commerce_line_items as $line_item_wrapper) {
      if (in_array($line_item_wrapper->type->value(), $line_item_types)) {
        $product_ids[] = (int)$line_item_wrapper->commerce_product->raw();
      }
    }
    return $product_ids;
  }

  /**
   * Return order products quantity.
   * @param integer|stdClass|EntityMetadataWrapper $order Order id, or object, or wrapper
   * @return integer
   */
  public static function getOrderQuantity($order, $types = array()) {
    if ($order = EntityHelper::getEntityWrapper($order, 'commerce_order')) {
      if (!$types) {
        $types = commerce_product_line_item_types();
      }
      return commerce_line_items_quantity($order->commerce_line_items, $types);
    }
  }

  /**
   * Return cart products quantity.
   * @return integer
   */
  public static function getCartQuantity() {
    $cart_wrapper = self::getCartOrderWrapper();
    return self::getOrderQuantity($cart_wrapper);
  }

  /**
   * Return order base price array.
   * @param integer|stdClass|EntityMetadataWrapper $order Order id, or object, or wrapper
   * @return array Price component array
   */
  public static function getOrderBasePrice($order) {
    $order = EntityHelper::getEntityWrapper($order, 'commerce_order');
    $total = $order->commerce_order_total->value();
    return commerce_price_component_total($total, 'base_price');
  }

  /**
   * Return shipping serive.
   * @param integer|stdClass|EntityMetadataWrapper $order Order id, or object, or wrapper
   */
  public static function getShippingService($order) {
    $order = EntityHelper::getEntityWrapper($order, 'commerce_order');
    foreach ($order->commerce_line_items as $delta => $line_item_wrapper) {
      if ($line_item_wrapper->type->value() == 'shipping') {
        return $line_item_wrapper->commerce_shipping_service->value();
      }
    }
  }

  /**
   * Return product display nid by product id.
   * @param integer $product_id Product id
   * @param string|NULL $bundle Bundle name
   * @param string $field_name Field name
   * @return integer|FALSE Product display nid or FALSE
   */
  public static function getProductDisplayNidByProductId($product_id, $bundle = 'product_display', $field_name = 'field_product') {
    static $cache = array();

    if (isset($cache[$product_id])) {
      return $cache[$product_id];
    }

    $query = db_select('field_data_' . $field_name, 'fp');
    $query->fields('fp', array('entity_id'));
    $query->condition('fp.' . $field_name . '_product_id', $product_id);
    if ($bundle) {
      $query->condition('fp.bundle', $bundle);
    }
    $query->range(0, 1);
    $product_display_nid = $query->execute()->fetchField();

    if ($product_display_nid) {
      $cache[$product_id] = $product_display_nid;
    }

    return $product_display_nid;
  }

  /**
   * Return product display nid by product sku.
   * @param string $product_sku Product SKU
   * @param string $bundle Bundle name
   * @param string $field_name Field name
   * @return integer|FALSE Product display nid or FALSE
   */
  public static function getProductDisplayNidByProductSku($product_sku, $bundle = 'product_display', $field_name = 'field_product') {
    $query = db_select('field_data_' . $field_name, 'fp');
    $query->fields('fp', array('entity_id'));
    $query->innerJoin('commerce_product', 'p', 'p.product_id = fp.' . $field_name . '_product_id');
    $query->condition('p.sku', $product_sku);
    $query->condition('fp.bundle', $bundle);
    $query->range(0, 1);
    return $query->execute()->fetchField();
  }

  /**
   * Return product display nid by line item
   * @param integer|stdClass|EntityMetadataWrapper $line_item Line item
   * @param string $bundle Bundle name
   * @param string $field_name Field name
   * @return integer|FALSE Product display nid or FALSE
   */
  public static function getProductDisplayNidByLineItem($line_item, $bundle = 'product_display', $field_name = 'field_product') {
    $line_item_wrapper = EntityHelper::getEntityWrapper($line_item, 'commerce_line_item');
    $product_id = $line_item_wrapper->commerce_product->raw();
    return self::getProductDisplayNidByProductId($product_id, $bundle, $field_name);
  }

  /**
   * Return product id by sku.
   * @param string $sku Product sku
   * @return integer|NULL Product id
   */
  public static function getProductIdBySku($sku) {
    $product_id = db_select('commerce_product', 'p')
      ->fields('p', array('product_id'))
      ->condition('sku', $sku)
      ->execute()
      ->fetchField();

    return $product_id ? (int)$product_id : NULL;
  }

  /**
   * Return product ids from product reference field.
   * @param integer|stdClass|EntityMetadataWrapper $product_display Product display nid, or object, or wrapper
   * @param string $field_name Field name
   * @return array
   */
  public static function getProductDisplayProductIds($product_display, $field_name = 'field_product') {
    return EntityHelper::getFieldValues($product_display, $field_name, 'product_id', 'node');
  }

  /**
   * Return TRUE if product id contains product.
   * @param integer|stdClass|EntityMetadataWrapper $product_display Product display nid, or object, or wrapper
   * @param integer $product_id Product id
   * @param string $field_name Field name
   * @return boolean
   */
  public static function productDisplayHasProduct($product_display, $product_id, $field_name = 'field_product') {
    return in_array($product_id, self::getProductDisplayProductIds($product_display, $field_name));
  }

  /**
   * Add product in product display reference field.
   * @param integer|stdClass|EntityMetadataWrapper $product_display Product display nid, or object, or wrapper
   * @param array $product_ids Product ids array
   * @param string $field_name Field name
   */
  public static function addProductsToProductDisplay($product_display, $product_ids, $field_name = 'field_product') {
    $product_display_product_ids = self::getProductDisplayProductIds($product_display, $field_name);
    if ($product_ids = array_diff($product_ids, $product_display_product_ids)) {
      /** @var EntityDrupalWrapper $product_display_wrap */
      $product_display_wrap = EntityHelper::getEntityWrapper($product_display, 'node');
      foreach ($product_ids as $product_id) {
        $product_display_wrap->{$field_name}[] = $product_id;
      }
      $product_display_wrap->save();
    }
  }

  /**
   * Return formatted price amount.
   * @param array|integer $price Price array or price amount
   * @param boolean $abs Set TRUE if you want to use abs() for price amount
   * @param boolean $amount_is_decimal Set TRUE if price amount is decimal
   * @return string
   */
  public static function getFormattedPrice($price, $abs = FALSE, $amount_is_decimal = FALSE) {
    if (is_array($price)) {
      $amount = $price['amount'];
      $currency_code = $price['currency_code'];
    }
    else {
      $amount = $price;
      $currency_code = commerce_default_currency();
    }

    if ($amount_is_decimal) {
      $amount = commerce_currency_decimal_to_amount($amount, $currency_code);
    }
    if ($abs) {
      $amount = abs($amount);
    }

    return commerce_currency_format($amount, $currency_code);
  }

  /**
   * Return decimal price amount.
   * @param array|integer $price Price array or price amount
   * @return float
   */
  public static function getDecimalAmount($price, $round = FALSE) {
    if (is_array($price)) {
      $amount = $price['amount'];
      $currency_code = $price['currency_code'];
    }
    else {
      $amount = $price;
      $currency_code = commerce_default_currency();
    }

    $decimal_amount = commerce_currency_amount_to_decimal($amount, $currency_code);

    if ($round) {
      $decimal_amount = commerce_round(COMMERCE_ROUND_HALF_UP, $decimal_amount);
    }

    return $decimal_amount;
  }

  /**
   * Return product sale count.
   * @param integer $product_id Product ID
   * @param array|NULL $conditions Array of conditions
   * @return integer
   */
  public static function getProductSaleCount($product_id, $conditions = NULL) {
    $query = db_select('field_data_commerce_product', 'lp');
    $query->addExpression('COUNT(*)');
    $query->innerJoin('commerce_line_item', 'l', 'l.line_item_id = lp.entity_id');
    $query->innerJoin('commerce_order', 'o', 'o.order_id = l.order_id');
    $query->condition('o.status', 'completed');
    $query->condition('lp.commerce_product_product_id', $product_id);
    if ($conditions) {
      foreach ($conditions as $condition) {
        $operator = isset($condition['operator']) ? $condition['operator'] : NULL;
        $query->condition($condition['field'], $condition['value'], $operator);
      }
    }
    return (int)$query->execute()->fetchField();
  }

  /**
   * Return product display sale count.
   * @param integer $product_display_nid Product display nid
   * @param array|NULL $conditions Array of conditions
   * @return integer
   */
  public static function getProductDisplaySaleCount($product_display_nid, $conditions = NULL) {
    $query = db_select('node', 'n');
    $query->addExpression('COUNT(*)');
    $query->innerJoin('field_data_field_product', 'fp', 'fp.entity_id = n.nid');
    $query->innerJoin('field_data_commerce_product', 'lp', 'lp.commerce_product_product_id = fp.field_product_product_id');
    $query->innerJoin('commerce_line_item', 'l', 'l.line_item_id = lp.entity_id');
    $query->innerJoin('commerce_order', 'o', 'o.order_id = l.order_id');
    $query->condition('o.status', 'completed');
    $query->condition('n.nid', $product_display_nid);
    if ($conditions) {
      foreach ($conditions as $condition) {
        $operator = isset($condition['operator']) ? $condition['operator'] : NULL;
        $query->condition($condition['field'], $condition['value'], $operator);
      }
    }
    return (int)$query->execute()->fetchField();
  }

  /**
   * Return products buyed with $product_id.
   * @param integer|integer[] $product_id Product id
   * @param integer|NULL $limit Limit
   * @return array Key - product id, Value - buyed count
   */
  public static function getSimilarProductSales($product_id, $limit = 100) {
    $query = db_select('commerce_order', 'o');
    $query->addField('p', 'product_id');
    $query->addExpression('COUNT(*)', 'cnt');
    $query->innerJoin('commerce_line_item', 'l1', 'l1.order_id = o.order_id');
    $query->innerJoin('commerce_line_item', 'l2', 'l2.order_id = o.order_id');
    $query->innerJoin('field_data_commerce_product', 'lp1', 'lp1.entity_id = l1.line_item_id');
    $query->innerJoin('field_data_commerce_product', 'lp2', 'lp2.entity_id = l2.line_item_id');
    $query->innerJoin('commerce_product', 'p', 'p.product_id = lp2.commerce_product_product_id');
    $query->condition('o.status', 'completed');
    $query->condition('p.status', 1);
    $query->condition('lp1.commerce_product_product_id', $product_id);
    $query->where('lp1.commerce_product_product_id <> lp2.commerce_product_product_id');
    $query->groupBy('p.product_id');
    $query->orderBy('cnt', 'DESC');
    $query->orderBy('p.product_id', 'DESC');
    if ($limit) {
      $query->range(0, $limit);
    }
    return $query->execute()->fetchAllKeyed();
  }

  /**
   * Return product displays buyed with $product_display_nid.
   * @param integer|integer[] $product_display_nid Product display nid
   * @param integer|NULL $limit Limit
   * @return array Key - product display nid, Value - buyed count
   */
  public static function getSimilarProductDisplaySales($product_display_nid, $limit = 100, $query_alter_callback = NULL) {
    $query = db_select('commerce_order', 'o');
    $query->addField('n', 'nid');
    $query->addExpression('COUNT(*)', 'cnt');
    $query->innerJoin('commerce_line_item', 'l1', 'l1.order_id = o.order_id');
    $query->innerJoin('commerce_line_item', 'l2', 'l2.order_id = o.order_id');
    $query->innerJoin('field_data_commerce_product', 'lp1', 'lp1.entity_id = l1.line_item_id');
    $query->innerJoin('field_data_commerce_product', 'lp2', 'lp2.entity_id = l2.line_item_id');
    $query->innerJoin('field_data_field_product', 'fp1', 'fp1.field_product_product_id = lp1.commerce_product_product_id');
    $query->innerJoin('field_data_field_product', 'fp2', 'fp2.field_product_product_id = lp2.commerce_product_product_id');
    $query->innerJoin('node', 'n', 'n.nid = fp2.entity_id');
    $query->condition('o.status', 'completed');
    $query->condition('n.status', 1);
    $query->condition('fp1.entity_id', $product_display_nid);
    $query->where('fp1.entity_id <> fp2.entity_id');
    $query->groupBy('n.nid');
    $query->orderBy('cnt', 'DESC');
    $query->orderBy('n.nid', 'DESC');
    if ($limit) {
      $query->range(0, $limit);
    }
    if ($query_alter_callback && is_callable($query_alter_callback)) {
      $query_alter_callback($query);
    }
    return $query->execute()->fetchAllKeyed();
  }

  /**
   * Return last user order id.
   * @param integer $uid User id
   * @return integer|NULL
   */
  public static function getLastUserOrderId($uid = NULL) {
    if ($uid === NULL) {
      $uid = $GLOBALS['user']->uid;
    }
    if ($uid == 0) {
      return NULL;
    }

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'commerce_order');
    $query->propertyCondition('uid', $uid);
    $query->propertyCondition('status', array('cart', 'checkout_checkout'), 'NOT IN');
    $query->propertyOrderBy('order_id', 'DESC');
    $query->range(0, 1);
    $result = $query->execute();
    if ($result) {
      return current($result['commerce_order'])->order_id;
    }
  }

  /**
   * Return payment method title.
   * @param string $payment_method_instance_id Payment method instance id
   * @return string
   */
  public static function getPaymentMethodTitle($payment_method_instance_id) {
    list(, $rule_name) = explode('|', $payment_method_instance_id);
    $rule = rules_config_load($rule_name);
    return $rule->label;
  }

  /**
   * Return unique sku.
   * @param integer $sku_length SKU length
   * @return string
   */
  public static function getUniqueSku($sku_length = 6) {
    $sku = drupal_substr(md5(drupal_random_key()), 0, $sku_length);

    if (self::getProductIdBySku($sku)) {
      $sku = self::getUniqueSku($sku_length);
    }

    return $sku;
  }

  /**
   * Return TRUE if node type is product display.
   * @param string $node_type Node type
   * @return boolean
   */
  public static function nodeTypeIsProductDisplay($node_type) {
    static $product_display_node_types;

    if ($product_display_node_types === NULL) {
      $product_display_node_types = commerce_product_reference_node_types();
    }

    return isset($product_display_node_types[$node_type]);
  }

  /**
   * Return TRUE if line item type is product type.
   * @param string $line_item_type Line item type
   * @return boolean
   */
  public static function lineItemTypeIsProductType($line_item_type) {
    static $product_line_item_types;

    if ($product_line_item_types === NULL) {
      $product_line_item_types = commerce_product_line_item_types();
    }

    return in_array($line_item_type, $product_line_item_types);
  }

  /**
   * Return bundle product reference fields.
   * @param string $entity_type Entity type
   * @param string $bundle_name Bundle name
   * @return array
   */
  public static function getProductReferenceFieldsInfo($entity_type, $bundle_name) {
    static $all_product_reference_fields;

    if ($all_product_reference_fields === NULL) {
      $all_product_reference_fields = commerce_info_fields('commerce_product_reference', $entity_type);
    }

    $product_reference_fields = array();
    foreach ($all_product_reference_fields as $product_reference_field) {
      if (isset($product_reference_field['bundles'][$entity_type]) && array_search($bundle_name, $product_reference_field['bundles'][$entity_type]) !== FALSE) {
        $product_reference_fields[$product_reference_field['field_name']] = $product_reference_field;
      }
    }

    return $product_reference_fields;
  }
}
