<?php

class EntityHelper {
  /**
   * Return entities by conditions use EntityFieldQuery.
   * @param string $entity_type The entity type to load, e.g. node, user or taxonomy_term.
   * @param array|NULL $properties An associative array of properties values.
   * @param array|NULL $fields An associative array of fields values.
   * @param integer|NULL $limit Limit.
   * @return array|NULL EntityFieldQuery result array.
   *
   * Example:
   * @code
   * $nodes = EntityHelper::getEntitiesByConditions(
   *   'node',
   *   array('type' => 'article'),
   *   array('field_category' => array('tid' => 2))
   * );
   * @endcode
   */
  public static function getEntitiesByConditions($entity_type, $properties = NULL, $fields = NULL, $limit = NULL) {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', $entity_type);
    if ($properties) {
      foreach ($properties as $name => $value) {
        $query->propertyCondition($name, $value);
      }
    }
    if ($fields) {
      foreach ($fields as $name => $values) {
        foreach ($values as $column => $value) {
          $query->fieldCondition($name, $column, $value);
        }
      }
    }
    if ($limit) {
      $query->range(0, $limit);
    }
    $result = $query->execute();
    if ($result) {
      return $result[$entity_type];
    }
  }

  /**
   * Return entity by conditions use EntityFieldQuery.
   * @param string $entity_type The entity type to load, e.g. node, user or taxonomy_term.
   * @param array|NULL $properties An associative array of properties values.
   * @param array|NULL $fields An associative array of fields values.
   * @see EntityHelper::getEntitiesByConditions()
   */
  public static function getEntityByConditions($entity_type, $properties = NULL, $fields = NULL) {
    $entities = self::getEntitiesByConditions($entity_type, $properties, $fields, 1);
    if ($entities) {
      return current($entities);
    }
  }

  /**
   * Return field min/max value.
   * @param string $field_name Field name
   * @param string $value_name Value key name
   * @return object
   */
  public static function getFieldMinMaxValue($field_name, $value_name = 'value') {
    $col_name = $field_name . '_' . $value_name;
    $query = db_select('field_data_' . $field_name);
    $query->addExpression('MIN(' . $col_name . ')', 'min');
    $query->addExpression('MAX(' . $col_name . ')', 'max');
    return $query->execute()->fetchObject();
  }

  /**
   * Return new field collection item wrapper (without save).
   * @param string $field_name Field name
   * @param string $host_entity_type Host entity type
   * @param $host_entity Host entity object
   * @return EntityDrupalWrapper
   */
  public static function getNewFieldCollectionWrapper($field_name, $host_entity_type, $host_entity) {
    $collection_item = entity_create('field_collection_item', array('field_name' => $field_name));
    $collection_item->setHostEntity($host_entity_type, $host_entity);
    return entity_metadata_wrapper('field_collection_item', $collection_item);
  }

  /**
   * Return field first value.
   * @param integer|stdClass|EntityMetadataWrapper $entity Entity id, or object, or wrapper
   * @param string $field_name Field name
   * @param mixed $default_value Default value
   * @param string|NULL $field_value_name Value column name or NULL, if you have return item array
   * @param string $entity_type Entity type
   * @return mixed
   */
  public static function getFieldValue($entity, $field_name, $default_value = NULL, $field_value_name = 'value', $entity_type = 'node') {
    $entity = self::getEntityObject($entity, $entity_type);
    $items = field_get_items($entity_type, $entity, $field_name);
    if ($items) {
      $first_item = current($items);
      return $field_value_name ? $first_item[$field_value_name] : $first_item;
    }
    return $default_value;
  }

  /**
   * Return field values.
   * @param integer|stdClass|EntityMetadataWrapper $entity Entity id, or object, or wrapper
   * @param string $field_name Field name
   * @param string|NULL $field_value_name Value column name or NULL, if you have return item array
   * @param string $entity_type Entity type
   * @return array
   */
  public static function getFieldValues($entity, $field_name, $field_value_name = 'value', $entity_type = 'node') {
    $entity = self::getEntityObject($entity, $entity_type);
    $values = array();

    if (!$entity) {
      return $values;
    }

    $field_items = field_get_items($entity_type, $entity, $field_name);

    if ($field_items) {
      $values = ArrayHelper::getColumn($field_items, $field_value_name);
    }

    return $values;
  }

  /**
   * Return field reference target IDs.
   * @param integer|stdClass|EntityMetadataWrapper $entity Owner entity id, or object, or wrapper
   * @param string $field_reference_name Field reference name
   * @param string $owner_entity_type Owner entity type
   * @return array
   */
  public static function getFieldReferenceTargetIds($entity, $field_name, $owner_entity_type = 'node') {
    return self::getFieldValues($entity, $field_name, 'target_id', $owner_entity_type);
  }

  /**
   * Return entity object.
   * @param integer|stdClass|EntityMetadataWrapper $entity Entity id, or object, or wrapper
   * @param string $entity_type Entity type
   * @return stdClass|NULL
   */
  public static function getEntityObject($entity, $entity_type) {
    if (is_numeric($entity)) {
      $entities = entity_load($entity_type, array($entity));
      return reset($entities);
    }
    if ($entity instanceof EntityMetadataWrapper) {
      return $entity->value();
    }
    if ($entity instanceof stdClass) {
      return $entity;
    }
  }

  /**
   * Return entity metadata wrapper.
   * @param integer|stdClass|EntityMetadataWrapper $entity Entity id, or object, or wrapper
   * @param string $entity_type Entity type
   * @return EntityMetadataWrapper|NULL
   */
  public static function getEntityWrapper($entity, $entity_type = 'node') {
    if (is_numeric($entity) || $entity instanceof stdClass) {
      return entity_metadata_wrapper($entity_type, $entity);
    }
    if ($entity instanceof EntityMetadataWrapper) {
      return $entity;
    }
  }

  /**
   * Return TRUE if field is not empty.
   * @param integer|stdClass|EntityMetadataWrapper $entity Entity id, or object, or wrapper
   * @param string $field_name Field name
   * @param string $entity_type Entity type
   * @param boolean $filter_empty_values Set TRUE if you allow filter empty values
   * @return boolean
   */
  public static function fieldNotEmpty($entity, $field_name, $entity_type = 'node', $filter_empty_values = FALSE) {
    $entity = self::getEntityObject($entity, $entity_type);
    $field_items = field_get_items($entity_type, $entity, $field_name);
    if (!empty($field_items)) {
      if ($filter_empty_values) {
        $field_definition = field_info_field($field_name);
        $field_filtered_items = _field_filter_items($field_definition, $field_items);
        return count($field_filtered_items) > 0;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Update field value without entity_save().
   * @param stdClass $entity Entity object
   * @param string $field_name Updated field name
   * @param string $entity_type Entity type
   */
  public static function fastUpdateFieldValue($entity, $field_name, $entity_type = 'node') {
    static $fields_id;

    if (!isset($fields_id[$field_name])) {
      $field_info = field_info_field($field_name);
      $fields_id[$field_name] = $field_info['id'];
    }

    field_sql_storage_field_storage_write($entity_type, $entity, 'update', array($fields_id[$field_name]));
  }

  /**
   * Return field allowed values.
   * @param string $field_name Field name
   * @return string
   */
  public static function getFieldAllowedValues($field_name) {
    $field = field_info_field($field_name);
    return $field['settings']['allowed_values'];
  }

  /**
   * Return file field upload directory.
   * @param string $field_name Field name
   * @param string $entity_type Entity type
   * @param string $bundle_name Bundle name
   * @return string
   */
  public static function getFileFieldDirectory($field_name, $entity_type, $bundle_name) {
    static $cache = array();

    $cache_key = $field_name . ':' . $entity_type . ':' . $bundle_name;
    if (!isset($cache[$cache_key])) {
      $field = field_info_field($field_name);
      $field_instance = field_info_instance($entity_type, $field_name, $bundle_name);
      $cache[$cache_key] = file_field_widget_uri($field, $field_instance);
    }

    return $cache[$cache_key];
  }

  /**
   * Return TRUE of Bundle has field.
   * @param string $entity_type Entity type
   * @param string $bundle_name Bundle name
   * @param string $field_name Field name
   * @return boolean
   */
  public static function bundleHasField($entity_type, $bundle_name, $field_name) {
    static $cache = array();

    $cache_key = $entity_type . ':' . $bundle_name . ':' . $field_name;

    if (!isset($cache[$cache_key])) {
      $cache[$cache_key] = (bool)field_info_instance($entity_type, $field_name, $bundle_name);
    }

    return $cache[$cache_key];
  }
}
