<?php

class NodeHelper {
  /**
   * Return TRUE if current page is node page.
   * @param string $node_type Node type
   * @return boolean
   */
  public static function isNodePage($node_type = NULL) {
    static $current_node_type;

    if ($current_node_type === NULL) {
      $menu_item = menu_get_item();
      if ($menu_item && $menu_item['path'] == 'node/%' && is_array($menu_item['page_arguments'])) {
        $current_node_type = $menu_item['page_arguments'][0]->type;
      }
      else {
        $current_node_type = '';
      }
    }

    if ($current_node_type) {
      if ($node_type) {
        return $node_type == $current_node_type;
      }
      else {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Unpublish node.
   * @param integer|stdClass|EntityMetadataWrapper $node Node nid, or object, or wrapper
   * @return boolean
   */
  public static function unpublishNode($node) {
    $node = EntityHelper::getEntityObject($node, 'node');
    if ($node->status != NODE_NOT_PUBLISHED) {
      $node->status = NODE_NOT_PUBLISHED;
      node_save($node);
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Publish node.
   * @param integer|stdClass|EntityMetadataWrapper $node Node nid, or object, or wrapper
   * @return boolean
   */
  public static function publishNode($node) {
    $node = EntityHelper::getEntityObject($node, 'node');
    if ($node->status != NODE_PUBLISHED) {
      $node->status = NODE_PUBLISHED;
      node_save($node);
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Fast unpublish node.
   * @param integer $nid Node nid
   * @param integer|NULL $vid Node vid
   * @param boolean $check_status Set TRUE if need check node status before update tables
   * @return boolean
   */
  public static function fastUnpublishNode($nid, $vid = NULL, $check_status = FALSE) {
    if ($check_status) {
      $node_status = db_select('node', 'n')
        ->fields('n', array('status'))
        ->condition('n.nid', $nid)
        ->execute()
        ->fetchField();
      if ($node_status == NODE_NOT_PUBLISHED) {
        return FALSE;
      }
    }

    // {node}
    $query = db_update('node');
    $query->fields(array('status' => NODE_NOT_PUBLISHED));
    $query->condition('nid', $nid);
    if ($vid) {
      $query->condition('vid', $vid);
    }
    $query->execute();

    // {node_revision}
    $query = db_update('node_revision');
    $query->fields(array('status' => NODE_NOT_PUBLISHED));
    $query->condition('nid', $nid);
    if ($vid) {
      $query->condition('vid', $vid);
    }
    $query->execute();

    return TRUE;
  }

  /**
   * Return node by title.
   * @param string $node_title Node title
   * @param string $node_type Node type
   * @return stdClass|FALSE Raw db row object
   */
  public static function getNodeByTitle($node_title, $node_type) {
    return db_select('node', 'n')
      ->fields('n')
      ->condition('n.title', $node_title)
      ->condition('n.type', $node_type)
      ->range(0, 1)
      ->execute()
      ->fetchObject();
  }

  /**
   * Return html link to node.
   * @param integer|stdClass|EntityMetadataWrapper $node Node nid, or object, or wrapper
   * @param boolean $absolute Set TRUE if needed absoulte link url
   * @return string
   */
  public static function getNodeLink($node, $absolute = FALSE) {
    $node = EntityHelper::getEntityObject($node, 'node');
    return l($node->title, 'node/' . $node->nid, array('absolute' => $absolute));
  }

  /**
   * Return node title use db query.
   * @param integer|integer[] $nid Node nid or nids array
   * @return string|string[] Node title or titles array
   */
  public static function getNodeTitle($nid) {
    $result = db_select('node', 'n')
      ->fields('n', array('nid', 'title'))
      ->condition('n.nid', $nid)
      ->execute()
      ->fetchAllKeyed();
    return is_numeric($nid) ? current($result) : $result;
  }

  /**
   * Create node url alias.
   * @param integer $nid Node nid
   * @param string $alias Node alias
   * @return boolean
   */
  public static function createPathAlias($nid, $alias) {
    $node = EntityHelper::getEntityObject($nid, 'node');
    $current_path = path_load(array('source' => 'node/' . $nid));

    if (!$current_path || $current_path['alias'] != $alias) {
      $new_path = array(
        'alias' => $alias,
        'source' => 'node/' . $nid,
        'language' => $node->language,
        'pid' => $current_path ? $current_path['pid'] : NULL,
      );
      path_save($new_path);

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Return node path alias or NULL if alias nor exists.
   * @param integer $nid Node nid
   * @return string|NULL
   */
  public static function getPathAlias($nid) {
    $path = 'node/' . $nid;
    $path_alias = drupal_get_path_alias($path);

    return ($path_alias != $path) ? $path_alias : NULL;
  }

  /**
   * Return node nid by node normal path or path alias.
   * @param string $path Node normal path or path alias.
   * @return integer
   */
  public static function getNodeNidByPath($path) {
    $node_normal_path = drupal_get_normal_path($path);
    return (int)arg(1, $node_normal_path);
  }

  /**
   * Return rendered nodes by nids.
   * @param array $nids Nids
   * @return string
   */
  public static function viewMultiple($nids, $view_mode = 'teaser') {
    $nodes = node_load_multiple($nids);
    $build = node_view_multiple($nodes, $view_mode);
    return drupal_render($build);
  }
}
