<?php

class TaxonomyHelper {
  /**
   * Return all term child tids (version 1). For large vocabularies.
   * @param integer $tid Term tid
   * @param boolean $with_current
   * @return array
   */
  public static function getAllChildTermTidsUseDb($tid, $with_current = FALSE) {
    static $cache = array();

    $childrens = array();

    if (is_numeric($tid) && isset($cache[$tid])) {
      $childrens = $cache[$tid];
    }
    else {
      $query = db_select('taxonomy_term_data', 't');
      $query->fields('t', array('tid'));
      $query->innerJoin('taxonomy_term_hierarchy', 'h', 'h.tid = t.tid');
      $query->condition('h.parent', $tid);
      $result = $query->execute()->fetchCol();

      $childrens = $result ? array_merge($result, self::getAllChildTermTidsUseDb($result)) : array();

      if (is_numeric($tid)) {
        $cache[$tid] = $childrens;
      }
    }

    if ($with_current) {
      $childrens_with_current = $childrens;
      array_unshift($childrens_with_current, $tid);
      return $childrens_with_current;
    }
    else {
      return $childrens;
    }
  }

  /**
   * Return all term child tids (version 2). For small vocabularies.
   * @param integer $tid Term tid
   * @param integer $vid Vocabulary vid
   * @param boolean $add_tid_in_result
   * @return array
   */
  public static function getAllChildTermTidsUseTree($tid, $vid, $add_tid_in_result = FALSE) {
    $child_tids = array();
    foreach (taxonomy_get_tree($vid, $tid) as $terms) {
      $child_tids[] = (int)$terms->tid;
    }
    if ($add_tid_in_result) {
      array_unshift($child_tids, $tid);
    }
    return $child_tids;
  }

  /**
   * Return parent term tid.
   * @param integer $tid Term tid
   * @return integer
   */
  public static function getParentTermTid($tid) {
    if ($parents = taxonomy_get_parents($tid)) {
      return (int)current($parents)->tid;
    }
    return 0;
  }

  /**
   * Return parent term object.
   * @param integer $tid Term tid
   * @return object
   */
  public static function getParentTerm($tid) {
    if ($parent_tid = self::getParentTermTid($tid)) {
      return taxonomy_term_load($parent_tid);
    }
  }

  /**
   * Return root parent term tid. If term no parent then return same tid.
   * @param integer $tid Term tid
   * @return integer
   */
  public static function getRootParentTermTid($tid) {
    if ($parents = taxonomy_get_parents_all($tid)) {
      return (int)end($parents)->tid;
    }
  }

  /**
   * Return prev or next term.
   * @param integer|stdClass|EntityMetadataWrapper $term Term
   * @param string $sibling_type Sibling type - 'prev' or 'next'
   * @param boolean $only_same_parent TRUE - if you have get term with same parent
   * @return object|FALSE Raw db row object
   */
  public static function getSiblingTerm($term, $sibling_type, $only_same_parent = TRUE) {
    $term = EntityHelper::getEntityObject($term, 'taxonomy_term');

    if ($sibling_type == 'next') {
      $operator = '>';
      $direction = 'ASC';
    }
    else {
      $operator = '<';
      $direction = 'DESC';
    }

    $query = db_select('taxonomy_term_data', 't');
    $query->fields('t', array('tid', 'name'));
    $query->condition('t.vid', $term->vid);
    $query->where("t.weight $operator :weight OR (t.weight = :weight AND t.name $operator :name)", array(
      ':weight' => $term->weight,
      ':name' => $term->name,
    ));
    $query->orderBy('t.weight', $direction);
    $query->orderBy('t.name', $direction);
    $query->range(0, 1);

    if ($only_same_parent) {
      $query->innerJoin('taxonomy_term_hierarchy', 'h', 'h.tid = t.tid');
      $query->condition('h.parent', self::getParentTermTid($term->tid));
    }

    return $query->execute()->fetchObject();
  }

  /**
   * Return term tid by term name.
   * @param string $term_name Term name
   * @param integer|NULL $vocabulary_vid Vocabulary vid
   * @param integer|NULL $parent_tid Parent term tid
   * @param boolean $create_nonexistent Set TRUE if you have create non-existent term
   * @return integer|NULL
   */
  public static function getTermTidByName($term_name, $vocabulary_vid = NULL, $parent_tid = NULL, $create_nonexistent = FALSE) {
    static $cache = array();

    $term_name = trim($term_name);
    if (!$term_name) {
      return NULL;
    }

    $cache_key = $term_name . ':' . $vocabulary_vid . ':' . $parent_tid;

    if (!isset($cache[$cache_key])) {
      $query = db_select('taxonomy_term_data', 't');
      $query->fields('t', array('tid'));
      $query->condition('t.name', $term_name);
      $query->range(0, 1);
      if ($vocabulary_vid) {
        $query->condition('t.vid', $vocabulary_vid);
      }
      if ($parent_tid !== NULL) {
        $query->innerJoin('taxonomy_term_hierarchy', 'h', 'h.tid = t.tid');
        $query->condition('h.parent', $parent_tid);
      }
      $tid = $query->execute()->fetchField();

      if (!$tid && $create_nonexistent) {
        $tid = TaxonomyHelper::createTerm($term_name, $vocabulary_vid, $parent_tid, FALSE);
      }

      if ($tid) {
        $cache[$cache_key] = (int)$tid;
      }
    }

    return isset($cache[$cache_key]) ? $cache[$cache_key] : NULL;
  }

  /**
   * Return term tid by term name hierarchy.
   * @paran array $names Names of terms
   * @paran integer|NULL $vocabulary_vid Vocabulary vid
   * @return integer|NULL
   */
  public static function getTermTidByNameHierarchy($names, $vocabulary_vid = NULL) {
    $tid = 0;
    foreach ($names as $term_name) {
      if (!$tid = self::getTermTidByName($term_name, $vocabulary_vid, $tid)) {
        return NULL;
      }
    }
    return (int)$tid;
  }

  /**
   * Return term name by tid use database query.
   * @param integer $tid Term tid
   * @return string|FALSE
   */
  public static function getTermName($tid) {
    static $cache;

    if (!$tid) {
      return NULL;
    }
    if (isset($cache[$tid])) {
      return $cache[$tid];
    }

    $term_name = db_query("
      SELECT t.name
      FROM {taxonomy_term_data} t
      WHERE t.tid = :tid
    ", array(':tid' => $tid))->fetchField();

    if ($term_name) {
      $cache[$tid] = $term_name;
    }

    return $term_name;
  }

  /**
   * Return term depth.
   * @param integer $tid Term tid
   * @return integer
   */
  public static function getTermDepth($tid) {
    return count(taxonomy_get_parents_all($tid));
  }

  /**
   * Return TRUE if term has children.
   * @param integer $tid Term tid
   * @return boolean
   */
  public static function termHasChildren($tid) {
    static $cache = array();

    if (!isset($cache[$tid])) {
      $query = db_select('taxonomy_term_data', 't');
      $query->fields('t', array('tid'));
      $query->innerJoin('taxonomy_term_hierarchy', 'h', 'h.tid = t.tid');
      $query->condition('h.parent', $tid);
      $query->range(0, 1);
      $cache[$tid] = (bool)$query->execute()->fetchField();
    }

    return $cache[$tid];
  }

  /**
   * Return tree item list with links on terms.
   * @param integer $vocabulary_vid Vocabulary vid
   * @param integer $parent_tid Parent term tid
   * @param integer|NULL $max_depth Maximum tree depth
   * @return array
   */
  public static function getTermsTreeItemList($vocabulary_vid, $parent_tid = 0, $max_depth = NULL) {
    $terms = taxonomy_get_tree($vocabulary_vid, $parent_tid, $max_depth);
    return array(
      '#theme' => 'item_list',
      '#items' => self::getTermsTreeItemListItems($terms, $parent_tid),
    );
  }

  /**
   * Return items for tree item list.
   * @return array
   */
  private static function getTermsTreeItemListItems($terms, $parent_tid = 0) {
    $items = array();

    foreach ($terms as $term) {
      if (in_array($parent_tid, $term->parents)) {
        $items[] = array(
          'data' => l($term->name, 'taxonomy/term/' . $term->tid),
          'children' => self::getTermsTreeItemListItems($terms, $term->tid),
        );
      }
    }

    return $items;
  }

  /**
   * Return terms node counter.
   * @return array
   */
  public static function getTermsNodeCounter($vocabulary_vid, $childs_count = TRUE, $node_type = NULL) {
    $args = array(':vid' => $vocabulary_vid);
    if ($node_type) {
      $args[':type'] = $node_type;
    }

    $terms = db_query("
      SELECT t.tid, t.name, h.parent, (
        SELECT COUNT(*) FROM {taxonomy_index} i
        LEFT JOIN {node} n ON n.nid = i.nid
        WHERE i.tid = t.tid AND n.status = 1
        " . ($node_type ? "AND n.type = :type" : "") . "
      ) node_count
      FROM {taxonomy_term_data} t
      INNER JOIN {taxonomy_term_hierarchy} h ON h.tid = t.tid
      WHERE t.vid = :vid
      ORDER BY weight
    ", $args)->fetchAllAssoc('tid');

    // Set has_child flag
    foreach ($terms as $tid => $term) {
      if (isset($terms[$term->parent])) {
        $terms[$term->parent]->has_child = TRUE;
      }
      if (!isset($term->has_child)) {
        $terms[$tid]->has_child = FALSE;
      }
    }

    // Calculate node count in child terms
    if ($childs_count) {
      foreach ($terms as $tid => $term) {
        $terms[$tid]->childs_node_count = $term->has_child ? self::getTermsNodeCounterChilds($terms, $term->tid) : 0;
      }
    }

    return $terms;
  }

  /**
   * Return term childs node count.
   * @return integer
   */
  private static function getTermsNodeCounterChilds($terms, $parent_tid) {
    $node_count = 0;
    foreach ($terms as $tid => $term) {
      if ($term->parent == $parent_tid) {
        $node_count += $term->node_count;
        if ($term->has_child) {
          $node_count += self::getTermsNodeCounterChilds($terms, $tid);
        }
      }
    }
    return $node_count;
  }

  /**
   * Return terms tree with counter.
   * @param integer $vocabulary_vid Vocabulary vid
   * @param integer $parent_tid Parent term tid
   * @param integer $max_depth Maximum depth. Set 0 to without maximum depth
   * @param boolean $sum_childs TRUE - sum childs counter
   * @param string $node_type Node type
   * @param string $counter_place 'before' - place counter before link, 'after' - place counter after link, other string - item template
   * @return array
   */
  public static function getTermsTreeItemListWithNodeCounter($vocabulary_vid, $parent_tid = 0, $max_depth = 0, $sum_childs = TRUE, $node_type = NULL, $counter_place = 'after') {
    $terms = self::getTermsNodeCounter($vocabulary_vid, $sum_childs, $node_type);
    return array(
      '#theme' => 'item_list',
      '#items' => self::getTermsTreeItemListWithNodeCounterItems($terms, $parent_tid, $max_depth, $sum_childs, 1, $counter_place),
    );
  }

  /**
   * Return items for self::getTermsTreeWithCounter().
   * @return array
   */
  private static function getTermsTreeItemListWithNodeCounterItems($terms, $parent_tid = 0, $max_depth = 0, $sum_childs = TRUE, $current_depth = 1, $counter_place = 'after') {
    $items = array();

    foreach ($terms as $tid => $term) {
      if ($term->parent == $parent_tid) {
        $children = array();
        if ($term->has_child && ($max_depth == 0 || $current_depth < $max_depth)) {
          $children = self::getTermsTreeItemListWithNodeCounterItems($terms, $tid, $max_depth, $sum_childs, $current_depth + 1);
        }

        $node_count = $term->node_count;
        if ($sum_childs) {
          $node_count += $term->childs_node_count;
        }

        // Format data
        $link = l($term->name, 'taxonomy/term/' . $term->tid);
        $counter = '<span class="counter">' . $node_count . '</span>';
        $data = '';
        if ($counter_place == 'before') {
          $data = $counter . ' ' . $link;
        }
        elseif ($counter_place == 'after') {
          $data = $link . ' ' . $counter;
        }
        else {
          $data = format_string($counter_place, array(
            '!link' => $link,
            '!counter' => $node_count,
          ));
        }

        $items[] = array(
          'data' => $data,
          'children' => $children,
        );
      }
    }

    return $items;
  }

  /**
   * Return vocabulary vid by vocabulary machine name.
   * @param string $vocabulary_machine_name Vocabulary machine name
   * @return integer|NULL
   */
  public static function getVocabularyVidByMachineName($vocabulary_machine_name) {
    if ($vocabulary = taxonomy_vocabulary_machine_name_load($vocabulary_machine_name)) {
      return (int)$vocabulary->vid;
    }
  }

  /**
   * Return TRUE if current page is term page.
   * @param integer|string|NULL $vocabulary Vocabulary vid or machine name
   * @return boolean
   */
  public static function isTermPage($vocabulary = NULL) {
    static $current_vocabulary_vid, $current_vocabulary_machine_name;

    if ($current_vocabulary_vid === NULL) {
      $menu_item = menu_get_item();
      if ($menu_item && $menu_item['path'] == 'taxonomy/term/%' && is_array($menu_item['page_arguments'])) {
        $current_vocabulary_vid = $menu_item['page_arguments'][0]->vid;
        $current_vocabulary_machine_name = $menu_item['page_arguments'][0]->vocabulary_machine_name;
      }
      else {
        $current_vocabulary_vid = '';
        $current_vocabulary_machine_name = '';
      }
    }

    if ($current_vocabulary_vid) {
      if ($vocabulary) {
        if (is_integer($vocabulary)) {
          return $vocabulary == $current_vocabulary_vid;
        }
        elseif (is_string($vocabulary)) {
          return $vocabulary == $current_vocabulary_machine_name;
        }
      }
      else {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Return current page term.
   * @return object
   */
  public static function getCurrentTerm() {
    if (self::isTermPage()) {
      return taxonomy_term_load(arg(2));
    }
  }

  /**
   * Create term.
   * @param string $term_name Term name
   * @param integer $vocabulary_vid Vocabulary vid
   * @param integer $parent_tid Parent term tid
   * @param boolean $check_exist TRUE - if check term on exist
   * @return integer Term tid
   */
  public static function createTerm($term_name, $vocabulary_vid, $parent_tid = 0, $check_exist = TRUE) {
    $term_name = trim($term_name);
    if (!$term_name) {
      return NULL;
    }

    if ($check_exist && $tid = TaxonomyHelper::getTermTidByName($term_name, $vocabulary_vid, $parent_tid)) {
      return $tid;
    }
    else {
      $term = (object)array(
        'vid' => $vocabulary_vid,
        'name' => $term_name,
        'parent' => $parent_tid,
      );
      taxonomy_term_save($term);

      return (int)$term->tid;
    }
  }

  /**
   * Create terms hierarchy from terms array.
   * @param array $term_names Names of terms
   * @param integer $vocabulary_vid Vocabulary vid
   * @return array Key - term tid, Value - term name
   */
  public static function createTermsHierarchy($term_names, $vocabulary_vid) {
    $terms = array();
    $parent_tid = 0;
    foreach ($term_names as $term_name) {
      if (!$term_name) {
        continue;
      }

      $tid = TaxonomyHelper::getTermTidByName($term_name, $vocabulary_vid, $parent_tid, TRUE);
      $terms[$tid] = $term_name;
      $parent_tid = $tid;
    }

    return $terms;
  }

  /**
   * Return term hierarchy names.
   * @param integer $tid Term tid
   * @return array
   */
  public static function getTermHierarchyNames($tid) {
    static $cache = array();

    if (!array_key_exists($tid, $cache)) {
      $all_parents = taxonomy_get_parents_all($tid);
      $hierarchy_names = ArrayHelper::getColumn($all_parents, 'name', 'tid');
      $cache[$tid] = array_reverse($hierarchy_names, TRUE);
    }

    return $cache[$tid];
  }
}
